@@ -3,7 +3,6 @@ from __future__ import annotations
 import functools
 from collections import defaultdict
 from typing import (
-    TYPE_CHECKING,
     ClassVar,
     Generic,
     Literal,
@@ -14,33 +13,30 @@ from typing import (
 )
 
 import higlass_schema as hgs
-from pydantic import BaseModel
+from pydantic import RootModel
 
-import higlass._display as display
 import higlass._utils as utils
 
 __all__ = [
+    "CombinedTrack",
     "EnumTrack",
     "HeatmapTrack",
     "IndependentViewportProjectionTrack",
-    "CombinedTrack",
     "PluginTrack",
     "TrackT",
     "View",
     "ViewT",
     "Viewconf",
-    "concat",
-    "hconcat",
-    "vconcat",
-    "track",
-    "view",
     "combine",
+    "concat",
     "divide",
+    "hconcat",
     "lock",
+    "track",
+    "vconcat",
+    "view",
 ]
 
-if TYPE_CHECKING:
-    from higlass.server import TilesetResource
 
 ## Mixins
 
@@ -125,46 +121,16 @@ class _OptionsMixin:
         return track
 
 
-class _TilesetMixin:
-    def tileset(
-        self: TrackT,  # type: ignore
-        tileset: TilesetResource,
-        inplace: bool = False,
-    ) -> TrackT:  # type: ignore
-        """Replace or add a tileset to a Track.
-
-        A convenience method to update a track with a tileset.
-
-        Parameters
-        ----------
-        tileset : TilesetResource
-            A tileset resource returned from `hg.server`.
-
-        inplace : bool, optional
-            Whether to modify the existing track in place or return
-            a new track (default: `False`)
-
-        Returns
-        -------
-        track : A track with the bound tileset.
-
-        """
-        track = self if inplace else utils.copy_unique(self)
-        track.server = tileset.server
-        track.tilesetUid = tileset.tileset.uid
-        return track
-
-
 ## Extend higlass-schema classes
 
 
-class EnumTrack(hgs.EnumTrack, _OptionsMixin, _PropertiesMixin, _TilesetMixin):
+class EnumTrack(hgs.EnumTrack, _OptionsMixin, _PropertiesMixin):
     """Represents a generic track."""
 
     ...
 
 
-class HeatmapTrack(hgs.HeatmapTrack, _OptionsMixin, _PropertiesMixin, _TilesetMixin):
+class HeatmapTrack(hgs.HeatmapTrack, _OptionsMixin, _PropertiesMixin):
     """Represets a specialized heatmap track."""
 
     ...
@@ -174,20 +140,19 @@ class IndependentViewportProjectionTrack(
     hgs.IndependentViewportProjectionTrack,
     _OptionsMixin,
     _PropertiesMixin,
-    _TilesetMixin,
 ):
     """Represents a view-independent viewport projection track."""
 
     ...
 
 
-class CombinedTrack(hgs.CombinedTrack, _OptionsMixin, _PropertiesMixin, _TilesetMixin):
+class CombinedTrack(hgs.CombinedTrack, _OptionsMixin, _PropertiesMixin):
     """Represents a track combining multiple tracks."""
 
     ...
 
 
-class PluginTrack(hgs.BaseTrack, _OptionsMixin, _PropertiesMixin, _TilesetMixin):
+class PluginTrack(hgs.BaseTrack, _OptionsMixin, _PropertiesMixin):
     """Represents an unknown plugin track."""
 
     plugin_url: ClassVar[str]
@@ -380,16 +345,18 @@ class Viewconf(hgs.Viewconf[View[TrackT]], _PropertiesMixin, Generic[TrackT]):
     """Represents a top-level viewconfig, or a complete HiGlass visualization."""
 
     def _repr_mimebundle_(self, include=None, exclude=None):
-        """ "Displays the view config in an IPython environment."""
-        renderer = display.renderers.get()
-        plugin_urls = [] if self.views is None else gather_plugin_urls(self.views)
-        return renderer(self.dict(), plugin_urls=plugin_urls)
+        """Displays the view config in an IPython environment."""
+        return self.widget()._repr_mimebundle_()
 
     def widget(self, **kwargs):
         """Create a Jupyter Widget display for this view config."""
         from higlass._widget import HiGlassWidget
 
-        return HiGlassWidget(self.dict(), **kwargs)
+        return HiGlassWidget(
+            viewconf=self.model_dump(),
+            plugin_urls=[] if self.views is None else gather_plugin_urls(self.views),
+            **kwargs,
+        )
 
     @classmethod
     def from_url(cls, url: str, **kwargs):
@@ -416,7 +383,7 @@ class Viewconf(hgs.Viewconf[View[TrackT]], _PropertiesMixin, Generic[TrackT]):
         with urllib.urlopen(request) as response:
             raw = response.read()
 
-        return cls.parse_raw(raw)
+        return cls.model_validate_json(raw)
 
     def locks(
         self,
@@ -603,7 +570,7 @@ def concat(
         raise ValueError("concat method must be 'vertical' or 'horizontal'.")
 
     # gather views and adjust layout
-    views = [v.copy(deep=True) for v in b.views]
+    views = [v.model_copy(deep=True) for v in b.views]
     offset = 0 if a.views is None else max(map(mapper, a.views))
     for view in views:
         curr = getattr(view.layout, field)
@@ -616,7 +583,7 @@ def concat(
         locks = getattr(b, lockattr)
         if locks:
             if getattr(a, lockattr) is None:
-                setattr(a, lockattr, locks.copy(deep=True))
+                setattr(a, lockattr, locks.model_copy(deep=True))
             else:
                 getattr(a, lockattr).locksByViewUid.update(locks.locksByViewUid)
                 getattr(a, lockattr).locksDict.update(locks.locksDict)
@@ -634,7 +601,7 @@ vconcat = functools.partial(concat, "vertical")
 # TODO: register plugins globally to work here?
 
 
-class _TrackCreator(BaseModel):
+class _TrackCreator(RootModel):
     """Create track instances from their track type.
 
     Used internally by `hg.track` to leverage pydantic's ability to get
@@ -642,10 +609,10 @@ class _TrackCreator(BaseModel):
 
     Example:
     -------
-    >>> assert isinstance(_TrackCreator(type="heatmap").__root__, HeatmapTrack)
+    >>> assert isinstance(_TrackCreator(type="heatmap").root, HeatmapTrack)
     """
 
-    __root__: Track
+    root: Track
 
 
 @overload
@@ -688,7 +655,7 @@ def track(
     if uid is None:
         uid = utils.uid()
     data = dict(type=type_, uid=uid, **kwargs)
-    return _TrackCreator.parse_obj(data).__root__
+    return _TrackCreator.model_validate(data).root
 
 
 def view(
@@ -745,16 +712,16 @@ def view(
     if layout is None:
         layout = hgs.Layout(x=x, y=y, w=width, h=height)
     else:
-        layout = hgs.Layout(**layout.dict())
+        layout = hgs.Layout(**layout.model_dump())
 
     if tracks is None:
         data = defaultdict(list)
     else:
-        data = defaultdict(list, tracks.dict())
+        data = defaultdict(list, tracks.model_dump())
 
     for track in _tracks:
         if isinstance(track, hgs.Tracks):
-            track = track.dict()
+            track = track.model_dump()
             for position, track_list in track.items():
                 data[position].extend(track_list)
         else:
@@ -763,7 +730,7 @@ def view(
             else:
                 position = utils.track_default_position.get(track.type)
                 if position is None:
-                    raise ValueError("No default track type")
+                    raise ValueError(f"No position for track type: {track.type}")
 
             data[position].append(track)
 
@@ -806,8 +773,8 @@ def combine(t1: Track, t2: Track, uid: str | None = None, **kwargs) -> CombinedT
         uid = utils.uid()
 
     if isinstance(t1, CombinedTrack):
-        copy = CombinedTrack(**t1.dict())
-        copy.contents.append(t2.__class__(**t2.dict()))
+        copy = CombinedTrack(**t1.model_dump())
+        copy.contents.append(t2.__class__(**t2.model_dump()))
         for key, val in kwargs.items():
             setattr(copy, key, val)
         return copy
@@ -815,7 +782,7 @@ def combine(t1: Track, t2: Track, uid: str | None = None, **kwargs) -> CombinedT
     return CombinedTrack(
         type="combined",
         uid=uid,
-        contents=[track.__class__(**track.dict()) for track in (t1, t2)],
+        contents=[track.__class__(**track.model_dump()) for track in (t1, t2)],
         **kwargs,
     )
 
